/*
 * AnnotationUtil.java
 */

package com.example.tool;

import com.sun.javadoc.AnnotationDesc;
import com.sun.javadoc.AnnotationDesc.ElementValuePair;
import com.sun.javadoc.MethodDoc;

/**
 * AnnotationUtil is the utility to convert annotations to XDoclet tags.
 */
public class AnnotationUtil {

    /** Annotation type name: Transactional */
    private static final String TRANSACTIONAL =
            "org.springframework.transaction.annotation.Transactional";
    /** Annotation type name: RolesAllowed */
    private static final String ROLES_ALLOWED =
            "com.example.interceptor.security.RolesAllowed";
    /** Comment head. */
    private static final String COMMENT_HEAD = "\n     * ";

    /**
     * Converts annotations of the specified method to XDoclet tags.
     * @param methoddoc MethodDoc object.
     * @return XDoclet tags.
     */
    public static String convertToTags(MethodDoc methoddoc) {
        StringBuffer buf = new StringBuffer();
        AnnotationDesc annotations[] = methoddoc.annotations();
        for (int i = 0; i < annotations.length; i++) {
            String type = annotations[i].annotationType().qualifiedName();
            if (type.equals(TRANSACTIONAL)) {
                buf.append(convertToTransactionTag(annotations[i]));
            }
            else if (type.equals(ROLES_ALLOWED)) {
                buf.append(convertToPermissionTag(annotations[i]));
            }
        }
        return buf.toString();
    }
    
    /**
     * Converts @Transactional(propagation=Propagation.<type>)
     * to "@ejb.transaction type=<type>" tag.
     * @param annotation annotation description object.
     * @return XDoclet ejb.transaction tag.
     */
    private static String convertToTransactionTag(AnnotationDesc annotation) {
        ElementValuePair attrs[] = annotation.elementValues();
        String propagation = null;
        for (int i = 0; i < attrs.length; i++) {
            if (attrs[i].element().name().equals("propagation")) {
                propagation = attrs[i].value() == null ? "REQUIRED"
//                    : attrs[i].element().defaultValue().toString()
                    : attrs[i].value().toString();
                break;
            }
        }
        String trtype = null;
        if (propagation.equals("MANDATORY")) {
            trtype = "Mandatory";
        }
        else if (propagation.equals("NEVER")) {
            trtype = "Never";
        }
        else if (propagation.equals("NOT_SUPPORTED")) {
            trtype = "NotSupported";
        }
        else if (propagation.equals("REQUIRED")) {
            trtype = "Required";
        }
        else if (propagation.equals("REQUIRES_NEW")) {
            trtype = "RequiresNew";
        }
        else if (propagation.equals("SUPPORTS")) {
            trtype = "Supports";
        }
        else {
            throw new IllegalArgumentException("unsupported transaction "
                    + "attribute: " + propagation);
        }
        return COMMENT_HEAD + "@ejb.transaction type=\"" + trtype + "\"";
    }
    
    /**
     * Converts @RolesAllowed(<rolename>)
     * to "ejb.permission role-name=<rolename>" tag.
     * @annotation annotation description object.
     * @return XDoclet ejb.permission tag.
     */
    private static String convertToPermissionTag(AnnotationDesc annotation) {
        ElementValuePair attrs[] = annotation.elementValues();
        String role = null;
        for (int i = 0; i < attrs.length; i++) {
            if (attrs[i].element().name().equals("value")) {
                role = attrs[i].value().toString();
                break;
            }
        }
        return COMMENT_HEAD + "@ejb.permission role-name=" + role;
    }
}
